import logging
import pprint

import arrow

from .base import Port
from .config import ConfigMount
from .image import Image
from .network import Network
from .secret import SecretMount
from .volume import Mount

LOGGER = logging.getLogger(__name__)

class Endpoint():
    "The endpoint for a service"
    def __init__(self, ports, mode=None, virtual_ips=None):
        self.mode        = mode or 'vip'
        self.ports       = ports
        self.virtual_ips = virtual_ips

    def __eq__(self, other):
        return self.mode == other.mode and self.ports == other.ports

    @staticmethod
    def parse(payload):
        return Endpoint(
            ports       = [Port.parse(p) for p in payload.get('Ports', [])],
            mode        = payload.get('Spec', {}).get('Mode'),
            virtual_ips = payload.get('Spec', {}).get('VirtualIPs'),
        )

    def to_service_payload(self):
        return {
            'Mode'  : self.mode,
            'Ports' : [Port.to_service_payload(p) for p in self.ports],
        }

class NetworkAttachment():
    "An attachment of a service to a network"
    def __init__(self, network):
        self.network = network

    @staticmethod
    def parse(payload):
        return NetworkAttachment(network=Network(name=None, subnet=None, id_=payload['Target']))

    def to_service_payload(self):
        return {'Target': self.network.id_}

class Service():
    "A service in a docker swarm"
    def __init__(self, name, image, endpoint=None, environment=None, data=None, id_=None, previous=None, version=None, command=None, configs=None, constraints=None, log_name=None, log_options=None, mode='replicated', mounts=None, networks=None, replicas=1, secrets=None):
        self.command     = command
        self.configs     = configs or []
        self.constraints = constraints or []
        self.data        = data
        self.endpoint    = Endpoint.parse(endpoint) if isinstance(endpoint, str) else endpoint
        self.endpoint    = self.endpoint if self.endpoint else Endpoint([])
        self.environment = environment or {}
        self.id          = id_
        self.image       = Image.parse(image) if isinstance(image, str) else image
        self.log_name    = log_name
        self.log_options = log_options
        self.mode        = mode
        self.mounts      = mounts or []
        self.name        = name
        self.networks    = networks or []
        self.previous    = None
        self.replicas    = replicas
        self.secrets     = secrets or []
        self.version     = version

    def __eq__(self, other):
        if other is None:
            return False
        return all([getattr(self, prop) == getattr(other, prop) for prop in (
            'endpoint', 'image', 'mounts')])

    def __str__(self):
        return "Service {} {}".format(self.name, self.image)

    def __repr__(self):
        return str(self)

    async def create(self, client):
        payload = self.to_payload()
        return await client.service_create(payload)

    async def update(self, client):
        await client.service_update(
            self.old.id,
            self.old.version,
            payload,
        )

    def to_payload(self):
        result = {
            'EndpointSpec'      : self.endpoint.to_service_payload() if self.endpoint else None,
            'Mode'              : {},
            'Name'              : self.name,
            'TaskTemplate'      : {
                'ContainerSpec' : {
                    'Command'   : self.command,
                    'Configs'   : [c.to_service_payload() for c in self.configs],
                    'Env'       : ["{}={}".format(k, v) for k, v in self.environment.items()],
                    'Image'     : self.image.to_payload(),
                    'Mounts'    : [m.to_service_payload() for m in self.mounts],
                },
                'Networks'          : [n.to_service_payload() for n in self.networks],
                'Placement'         : {
                    'Constraints'   : self.constraints,
                },
            },
        }
        # Lame, docker can't handle empty arrays properly :(
        if self.secrets:
            result['TaskTemplate']['ContainerSpec']['Secrets'] = [s.to_service_payload() for s in self.secrets]
        if self.log_name:
            result['TaskTemplate']['LogDriver'] = {
                'Name'  : self.log_name,
            }
        if self.log_options:
            result['TaskTemplate']['LogDriver']['Options'] = self.log_options
        if self.mode == 'replicated':
            result['Mode']['Replicated'] = {'Replicas' : self.replicas}
        return result

    @staticmethod
    def parse(payload):
        previous = payload.pop('PreviousSpec', None)
        LOGGER.debug("Parsing %s", pprint.pformat(payload))
        task = payload['Spec']['TaskTemplate']
        return Service(
            command     = task['ContainerSpec'].get('Command'),
            configs     = ConfigMount.parse(task['ContainerSpec'].get('Configs')),
            constraints = task.get('Placement', {}).get('Constraints'),
            data        = payload,
            endpoint    = Endpoint.parse(payload['Endpoint']),
            environment = _parse_environment(payload),
            name        = payload['Spec']['Name'],
            id_         = payload['ID'],
            image       = task['ContainerSpec']['Image'],
            log_name    = task.get('LogDriver', {}).get('Name'),
            log_options = task.get('LogDriver', {}).get('Options'),
            mode        = 'replicated' if 'Replicated' in payload['Spec']['Mode'] else 'global',
            mounts      = [Mount.parse(p) for p in payload['Spec']['TaskTemplate']['ContainerSpec'].get('Mounts', [])],
            networks    = [NetworkAttachment.parse(n) for n in task.get('Networks', [])],
            previous    = previous,
            replicas    = payload['Spec']['Mode'].get('Replicated', {}).get('Replicas'),
            secrets     = SecretMount.parse(task['ContainerSpec'].get('Secrets')),
            version     = payload['Version']['Index'],
        )

def _parse_environment(payload):
    env = payload['Spec']['TaskTemplate']['ContainerSpec'].get('Env', [])
    pairs = [item.partition('=') for item in env]
    pairs = [(item[0], item[2]) for item in pairs]
    return dict(pairs)



import logging

LOGGER = logging.getLogger(__name__)

class Container():
    "A container"
    def __init__(self, image):
        self.id = None
        self.image = image
        self.warnings = None

        self.hostname = None
        self.domainname = None
        self.user = None
        self.attachstdin = False
        self.attachstdout = True
        self.attachstderr = True
        self.ports = []
        self.tty = False
        self.openstdin = False
        self.stdinonce = False
        self.env = []
        self.cmd = []
        self.healthcheck = None
        self.args_escaped = False
        self.volumes = []
        self.working_dir = None
        self.entrypoint = None
        self.network_disabled = False
        self.mac_address = None
        self.on_build = None
        self.labels = {}
        self.stop_signal = 'SIGTERM'
        self.stop_timeout = 10
        self.shell = None
        self.host_config = None
        self.networking_config = None

    def to_payload(self):
        return {
            #'Hostname'          : self.hostname,
            #'Domainname'        : self.domainname,
            #'User'              : self.user,
            #'AttachStdin'       : self.attachstdin,
            #'AttachStdout'      : self.attachstdout,
            #'AttachStderr'      : self.attachstderr,
            #'ExposedPorts'      : self.ports,
            #'Tty'               : self.tty,
            #'OpenStdin'         : self.openstdin,
            #'StdinOnce'         : self.stdinonce,
            #'Env'               : self.env,
            #'Cmd'               : self.cmd,
            #'Healthcheck'       : self.healthcheck,
            #'ArgsEscaped'       : self.args_escaped,
            'Binds':            ['{}:{}'.format(
                v.source,
                v.target,
            ) for v in self.volumes],
            'Image'             : self.image,
            'Volumes'           : {
                v.target   : {
                    'Name'    : v.source,
                } for v in self.volumes
            },
            #'WorkingDir'        : self.working_dir,
            'Entrypoint'        : self.entrypoint,
            #'NetworkDisabled'   : self.network_disabled,
            #'MacAddress'        : self.mac_address,
            #'OnBuild'           : self.on_build,
            #'Labels'            : self.labels,
            #'StopSignal'        : self.stop_signal,
            #'StopTimeout'       : self.stop_timeout,
            #'Shell'             : self.shell,
            #'HostConfig'        : self.host_config,
            #'NetworkingConfig'  : self.networking_config,
        }

    async def run(self, client):
        payload = self.to_payload()
        result = await client.container_run(payload)
        self.id = result['Id']
        self.warnings = result['Warnings']
        LOGGER.debug("Started container %s%s", self.id, ' '.join(self.warnings) if self.warnings else '')
        return self.id



from .base import Port
from .config import Config, ConfigMount
from .container import Container
from .image import Image
from .network import Network
from .secret import Secret, SecretMount
from .service import Endpoint, NetworkAttachment, Service
from .volume import Mount, Volume



import asyncio
import argparse
import logging
import pprint

import drophi.client

LOGGER = logging.getLogger(__name__)

def setup_log(level=logging.INFO):
    logging.basicConfig(level=level, format="[%(asctime)s] %(levelname)s pid:%(process)d %(name)s:%(lineno)d %(message)s")

def run():
    parser = argparse.ArgumentParser()
    parser.add_argument('-v', '--verbose', action='store_true', help="Show more verbose logging")

    subparsers = parser.add_subparsers(help='The command to perform')

    parser_ps = subparsers.add_parser('ps', help='Show the running containers')
    parser_ps.set_defaults(command=_ps)

    parser_events = subparsers.add_parser('events', help='Show events captured in real time')
    parser_events.set_defaults(command=_events)

    parser_service = subparsers.add_parser('service', help='Show information on docker swarm services')
    parser_service.add_argument('action', choices=['ls'], help='Show information on all services in the swarm')
    parser_service.set_defaults(command=_services)

    args = parser.parse_args()

    setup_log(level=logging.DEBUG if args.verbose else logging.INFO)

    if not hasattr(args, 'command'):
        LOGGER.info("You must select a sub-command")
        return 1

    client = drophi.client.Client()

    loop = asyncio.get_event_loop()
    try:
        result = loop.run_until_complete(args.command(args, client))
    except KeyboardInterrupt:
        result = 0
    loop.run_until_complete(client.close())
    loop.close()
    return result or 0

async def _ps(args, client):
    LOGGER.info("Running ps")
    pprint.pprint(await client.ps())

async def _events(args, client):
    async for event in client.streamevents():
        pprint.pprint(event.data)
        print('-'*20)

async def _services(args, client):
    if args.action == 'ls':
        services = await client.service_ls()
        for service in services:
            pprint.pprint(service.data)



import logging
import pprint

import drophi.types

LOGGER = logging.getLogger(__name__)

class Serializer():
    pass

class Mount(Serializer):
    @staticmethod
    def from_payload(payload):
        return drophi.types.Mount(
            source  = payload.get('Source'),
            target  = payload.get('Target'),
            type    = payload['Type'],
        )

    @staticmethod
    def to_payload(mount):
        return {
            'Source'    : mount.source.name,
            'Target'    : mount.target,
            'Type'      : mount.type,
        }



import io
import json
import logging
import pprint
import tarfile
import time
import urllib.parse

import aiohttp
import arrow
import drophi.types

LOGGER = logging.getLogger(__name__)

class CommunicationError(Exception):
    "Parent class for any communications errors with Docker"
    def __init__(self, method, url, status, text, payload=None, params=None):
        super().__init__("Failed to communicate with docker on {} {} with payload {} and params {}: status {} content {}".format(
            method, url, payload, params, status, text)
        )
        LOGGER.debug("Communcation error with docker on %s %s with payload %s\n and params %s: status %s content %s",
            method, url, pprint.pformat(payload), params, status, text)
        self.method  = method
        self.payload = payload
        self.params  = params
        self.status  = status
        self.text    = text
        self.url     = url

class Client():
    def __init__(self):
        self.connector = aiohttp.UnixConnector(path='/var/run/docker.sock')
        self.session = aiohttp.ClientSession(connector=self.connector)

    def _url(self, path, **kwargs):
        if kwargs:
            return 'http://docker/v1.30{}?{}'.format(path, '&'.join(["{}={}".format(k, v) for k, v in kwargs.items()]))
        else:
            return 'http://docker/v1.30{}'.format(path)

    async def _check_response(self, method, url, response, payload=None, params=None):
        if not (response.status >= 200 and response.status < 300):
            raise CommunicationError(
                method,
                url,
                response.status,
                await response.text(),
                payload,
                params,
            )

    async def _delete(self, path, query=None):
        url = self._url(path)
        query = query if query else {}
        LOGGER.debug("DELETE %s", path)
        async with self.session.delete(url, params=query) as response:
            await self._check_response('DELETE', url, response)

    async def _get(self, path, query=None):
        url = self._url(path)
        LOGGER.debug("GET %s", path)
        async with self.session.get(url, params=query) as response:
            await self._check_response('GET', url, response)
            if response.headers['Content-Type'] == 'application/json':
                data = await response.json()
            else:
                data = await response.read()
            return data

    async def _post(self, path, payload, query=None):
        url = self._url(path)
        query = query if query else {}
        LOGGER.debug("POST %s", url)
        async with self.session.post(url, json=payload, params=query) as response:
            await self._check_response('POST', url, response, payload, query)
            return await response.json()

    async def _put(self, path, payload, query=None):
        url = self._url(path)
        query = query if query else {}
        LOGGER.debug("PUT %s", url)
        async with self.session.put(url, json=payload, params=query) as response:
            await self._check_response('PUT', url, response)
            return await response.json()

    async def _putraw(self, path, data, query=None, headers=None):
        url = self._url(path)
        query = query if query else {}
        LOGGER.debug("PUT raw %s", url)
        async with self.session.put(url, data=data, params=query, headers=headers) as response:
            await self._check_response('PUT', url, response)
            result = await response.text()
            LOGGER.debug("PUT raw %s resulted in %s: %s", url, response.status, result)
            return result

    async def container_archive_get(self, container_id, path):
        try:
            result = await self._get( path=f'/containers/{container_id}/archive', query={'path': path},)
            buf = io.BytesIO(result)
            archive = tarfile.open(mode='r', fileobj=buf)
            return archive
        except CommunicationError as e:
            LOGGER.debug("Failed to get file archive: %s", e)
            if e.status == 404:
                raise FileNotFoundError(path)
            raise

    async def container_archive_put(self, container_id, path, archive, no_overwrite_dir_non_dir=True):
        return await self._putraw(
            path=f'/containers/{container_id}/archive',
            data=archive,
            query={'path': path}, #, 'noOverwriteDirNonDir': str(no_overwrite_dir_non_dir)},
            headers={'Content-Type': 'application/gzip'},
        )

    async def close(self):
        LOGGER.debug("Closing aiohttp session")
        await self.session.close()

    async def ps(self):
        return await self._get('/containers/json')

    async def config_create(self, payload):
        result = await self._post('/configs/create', payload)
        return result

    async def config_ls(self):
        data = await self._get('/configs')
        return [drophi.types.Config.parse(c) for c in data]

    async def config_rm(self, id_):
        return await self._delete(
            f'/configs/{id_}',
        )

    async def container_rm(self, id_, v=False, force=False, link=False):
        return await self._delete(
            f'/containers/{id_}',
            query={'v': str(v), 'force': str(force), 'link': str(link)},
        )

    async def container_run(self, payload):
        result = await self._post('/containers/create', payload)
        return result

    async def network_create(self, payload):
        result = await self._post('/networks/create', payload)
        return result

    async def network_ls(self):
        data = await self._get('/networks')
        return [drophi.types.Network.parse(n) for n in data]

    async def secret_create(self, payload):
        result = await self._post('/secrets/create', payload)
        return result

    async def secret_ls(self):
        data = await self._get('/secrets')
        return [drophi.types.Secret.parse(c) for c in data]

    async def secret_rm(self, id_):
        return await self._delete(
            f'/secrets/{id_}',
        )

    async def service_ls(self):
        data = await self._get('/services')
        return [drophi.types.Service.parse(s) for s in data]

    async def service_create(self, payload):
        LOGGER.debug("Creating service with %s", pprint.pformat(payload))
        return await self._post('/services/create', payload)

    async def service_update(self, id_, version, payload):
        LOGGER.debug("Updating service %s with version %s to %s", id_, version, pprint.pformat(payload))
        return await self._post(f'/services/{id_}/update', payload, query={'version': version})

    async def streamevents(self):
        LOGGER.debug("Subscribing to docker events")
        url = self._url('/events', since=time.time())
        data_buffer = b''
        text_buffer = ''
        async with self.session.get(url, timeout=0) as response:
            while True:
                data_buffer += await response.content.read(4096)
                try:
                    text_buffer += data_buffer.decode('utf-8')
                    data_buffer = b''
                except ValueError:
                    LOGGER.debug("Failed to decode data as UTF-8, waiting for more")
                    continue
                newline = text_buffer.find('\n')
                if newline:
                    message = text_buffer[:newline]
                    text_buffer = text_buffer[newline+1:]
                else:
                    LOGGER.debug("Waiting for more data to fine a newline and break off a message")
                    continue
                try:
                    data = json.loads(message)
                    yield Event(data)
                except ValueError:
                    LOGGER.error("Failed to parse JSON from '%s'", message)

    async def task_ls(self):
        LOGGER.debug("Listing swarm tasks")
        return await self._get('/tasks')

    async def volume_create(self, payload):
        return await self._post('/volumes/create', payload)

    async def volume_ls(self):
        data = await self._get('/volumes')
        return [drophi.types.Volume.parse(v) for v in data['Volumes']] if data['Volumes'] else []

    async def volume_rm(self, name, force=False):
        return await self._delete(f'/volumes/{name}', query={'force': str(force)})

class Event():
    def __init__(self, data):
        self.data = data

    def __str__(self):
        return "Docker Event {} {} {} at {}".format(
            self.type,
            self.action,
            self.id,
            self.time,
        )

    @property
    def type(self):
        return self.data['Type']

    @property
    def action(self):
        return self.data['Action']

    @property
    def id(self):
        return self.data['id']

    @property
    def time(self):
        return arrow.get(self.data['time'])



import arrow

class Image():
    "A container image"
    def __init__(self, name, tag, hashtype=None, hashvalue=None):
        self.hashtype  = hashtype
        self.hashvalue = hashvalue
        self.name      = name
        self.tag       = tag

    def __str__(self):
        if self.hashtype and self.hashvalue:
            return f'{self.name}:{self.tag}@{self.hashtype}:{self.hashvalue}'
        else:
            return f'{self.name}:{self.tag}'

    def __eq__(self, other):
        if isinstance(other, str):
            other = Image(other)

        return all([
            self.name == other.name,
            self.tag == other.tag,
        ])

    @classmethod
    def parse(cls, value):
        start, _, hash_ = value.partition('@')
        name, _, tag = start.partition(':')
        hashtype, _, hashvalue = hash_.partition(':')
        return cls(name, tag, hashtype or None, hashvalue or None)

    def to_payload(self):
        return f'{self.name}:{self.tag}' if self.tag else self.name




import arrow

class DockerObject():
    PROPERTIES = ()
    def __eq__(self, other):
        return all([getattr(self, p) == getattr(other, p) for p in self.PROPERTIES])

    def diff(self, other):
        return {
            p: (getattr(self, p), getattr(other, p))
            for p in self.PROPERTIES if getattr(self, p) != getattr(other, p)
        }

class Container():
    "A container"
    def __init__(self, image):
        self.image = image

        self.hostname = None
        self.domainname = None
        self.user = None
        self.attachstdin = False
        self.attachstdout = True
        self.attachstderr = True
        self.ports = []
        self.tty = False
        self.openstdin = False
        self.stdinonce = False
        self.env = []
        self.cmd = []
        self.healthcheck = None
        self.args_escaped = False
        self.volumes = []
        self.working_dir = None
        self.entrypoint = None
        self.network_disabled = False
        self.mac_address = None
        self.on_build = None
        self.labels = []
        self.stop_signal = 'SIGTERM'
        self.stop_timeout = 10
        self.shell = None
        self.host_config = None
        self.networking_config = None

    def get_create_payload(self):
        return {
            'Hostname'          : self.hostname,
            'Domainname'        : self.domainname,
            'User'              : self.user,
            'AttachStdin'       : self.attachstdin,
            'AttachStdout'      : self.attachstdout,
            'AttachStderr'      : self.attachstderr,
            'ExposedPorts'      : self.ports,
            'Tty'               : self.tty,
            'OpenStdin'         : self.opnstdin,
            'StdinOnce'         : self.stdinonce,
            'Env'               : self.env,
            'Cmd'               : self.cmd,
            'Healthcheck'       : self.healthcheck,
            'ArgsEscaped'       : self.args_escaped,
            'Image'             : self.image,
            'Volumes'           : [v.to_payload() for v in self.volumes],
            'WorkingDir'        : self.working_dir,
            'Entrypoint'        : self.entrypoint,
            'NetworkDisabled'   : self.network_disabled,
            'MacAddress'        : self.mac_address,
            'OnBuild'           : self.on_build,
            'Labels'            : self.labels,
            'StopSignal'        : self.stop_signal,
            'StopTimeout'       : self.stop_timeout,
            'Shell'             : self.shell,
            'HostConfig'        : self.host_config,
            'NetworkingConfig'  : self.networking_config,
        }

class Port():
    def __init__(self, name, published, target, protocol='tcp', mode='ingress'):
        self.name       = name
        self.protocol   = protocol
        self.mode       = mode
        self.published  = published
        self.target     = target

    def __eq__(self, other):
        return all([getattr(self, p) == getattr(other, p) for p in (
            'protocol', 'mode', 'published', 'target')])

    @staticmethod
    def parse(payload):
        return Port(
            mode        = payload['PublishMode'],
            name        = payload.get('Name'),
            protocol    = payload['Protocol'],
            published   = payload['PublishedPort'],
            target      = payload['TargetPort'],
        )

    def to_service_payload(self):
        return {
            'Name'              : self.name,
            'Protocol'          : self.protocol,
            'PublishedPort'     : self.published,
            'TargetPort'        : self.target,
        }




import contextlib
import imp
import os
import re
import subprocess

from setuptools import setup
from setuptools.command.sdist import sdist

DATA_ROOTS = []
PROJECT = 'drophi'
VERSION_FILE = 'drophi/__init__.py'

def _get_output_or_none(args):
    try:
        return subprocess.check_output(args).decode('utf-8').strip()
    except subprocess.CalledProcessError:
        return None
    except FileNotFoundError:
        return None

def _get_git_description():
    return _get_output_or_none(['git', 'describe'])

def _get_git_branches_for_this_commit():
    branches = _get_output_or_none(['git', 'branch', '-r', '--contains', 'HEAD'])
    split = branches.split('\n') if branches else []
    return [branch.strip() for branch in split]

def _is_on_releasable_branch(branches):
    return any([branch == 'origin/master' or branch.startswith('origin/hotfix') for branch in branches])

def _git_to_version(git):
    match = re.match(r'(?P<tag>[\d\.]+)-(?P<offset>[\d]+)-(?P<sha>\w{8})', git)
    if not match:
        version = git
    else:
        version = "{tag}.post0.dev{offset}".format(**match.groupdict())
    return version

@contextlib.contextmanager
def write_version():
    git_description = _get_git_description()
    git_branches = _get_git_branches_for_this_commit()
    version = _git_to_version(git_description) if git_description else None
    if git_branches and not _is_on_releasable_branch(git_branches):
        print("Forcing version to 0.0.1 because this commit is on branches {} and not a whitelisted branch".format(git_branches))
        version = '0.0.1'
    if not version:
        print("Not writing version, don't have a git repo")
        yield
        print("Done not writing version")
        return
    with open(VERSION_FILE, 'w') as version_file:
        version_file.write('__version__ = "{}"\n'.format(version))
        print("Wrote version {} to {}".format(version, VERSION_FILE))
    yield
    with open(VERSION_FILE, 'w') as version_file:
        version_file.write('__version__ = "development"\n')
        print("Wrote version development to {}".format(VERSION_FILE))

def _get_version_from_file():
    basedir = os.path.abspath(os.path.dirname(__file__))
    root = imp.load_source('__init__', os.path.join(basedir, PROJECT, '__init__.py'))
    return root.__version__

def _get_version_from_git():
    git_description = _get_git_description()
    git_branches = _get_git_branches_for_this_commit()
    version = _git_to_version(git_description) if git_description else None
    if git_branches and not _is_on_releasable_branch(git_branches):
        print("Forcing version to 0.0.1 because this commit is on branches {} and not a whitelisted branch".format(git_branches))
        version = '0.0.1'
    return version

def get_version():
    file_version = _get_version_from_file()
    git_version = _get_version_from_git()
    result = git_version if file_version == 'development' else file_version
    print("Got version {}".format(result))
    return result

def get_data_files():
    data_files = []
    for data_root in DATA_ROOTS:
        for root, _, files in os.walk(data_root):
            data_files.append((os.path.join(PROJECT, root), [os.path.join(root, f) for f in files]))
    return data_files

def main():
    with write_version():
        setup(
            name                 = "drophi",
            version              = get_version(),
            description          = "A client for interacting with docker daemon",
            url                  = "https://bitbucket.com/Authentise/drophi",
            long_description     = "A client for interacting with docker from Python",
            author               = "Authentise, Inc.",
            author_email         = "engineering@authentise.com",
            install_requires     = [
                'aiohttp>=2.0.7',
                'arrow>=0.10.0',
            ],
            extras_require       = {
                'develop': [
                    'pylint==1.6.4',
                    'sphinx==1.6.1',
                    'sphinx-autobuild==0.6.0',
                ]
            },
            packages             = [
                "drophi",
                "drophi.types",
            ],
            package_data         = {
                "drophi"         : ["drophi/*"],
                "drophi.types"   : ["drophi/types/*"],
            },
            data_files           = get_data_files(),
            scripts              = ['bin/drophi'],
            include_package_data = True,
        )

if __name__ == "__main__":
    main()



from .base import DockerObject

class Mount(DockerObject):
    PROPERTIES = ('readonly', 'source', 'target', 'type')
    def __init__(self, source, target, type='volume', readonly=True):
        self.readonly   = readonly
        self.source     = source
        self.target     = target
        self.type       = type

    def __eq__(self, other):
        return all([getattr(self, p) == getattr(other, p) for p in (
            'readonly', 'source', 'target', 'type')])

    def to_container_payload(self):
        return '{source}:{target}:{flag}'.format(
            source  = self.source,
            target  = self.target,
            flag    = 'ro' if self.readonly else 'rw',
        )

    def to_service_payload(self):
        return {
            'Target'        : self.target,
            'Source'        : self.source.name if isinstance(self.source, Volume) else self.source,
            'Type'          : self.type,
            'ReadOnly'      : self.readonly,
            'Consistency'   : 'default',
        }

    @classmethod
    def parse(cls, payload):
        return cls(
            readonly = payload.get('ReadOnly', False),
            source   = payload['Source'],
            target   = payload['Target'],
            type     = payload['Type'],
        )

class Volume(DockerObject):
    PROPERTIES = ('driver', 'labels', 'mountpoint', 'name', 'options', 'scope')
    def __init__(self, name, driver='local', labels=None, mountpoint=None, options=None, scope='local'):
        self.driver     = driver
        self.labels     = labels or {}
        self.mountpoint = mountpoint
        self.options    = options or {}
        self.name       = name
        self.scope      = scope

    def __str__(self):
        return f'Volume {self.name}'

    async def create(self, client):
        payload = self.to_payload()
        return await client.volume_create(payload)

    async def delete(self, client):
        return await client.volume_rm(self.name)

    @classmethod
    def parse(cls, payload):
        return cls(
            driver      = payload['Driver'],
            labels      = payload['Labels'] or {},
            mountpoint  = payload['Mountpoint'],
            name        = payload['Name'],
            options     = payload['Options'],
            scope       = payload['Scope'],
        )

    def to_payload(self):
        return {
            'Name'          : self.name,
            'Driver'        : 'local',
            'DriverOpts'    : {},
            'Labels'        : self.labels,
        }



from .base import DockerObject

class Network(DockerObject):
    "A docker network"
    def __init__(self, name, subnet, driver='bridge', id_=None):
        self.driver   = driver
        self.id_      = id_
        self.name     = name
        self.subnet   = subnet

    @staticmethod
    def parse(payload):
        return Network(
            id_         = payload['Id'],
            name        = payload['Name'],
            subnet      = payload['IPAM']['Config'][0]['Subnet'] if payload['IPAM']['Config'] else None,
        )

    def to_payload(self):
        return {
            'Name'              : self.name,
            'CheckDuplicate'    : True,
            'Driver'            : self.driver,
            'IPAM'              : {
                'Driver'        : 'default',
                'Config'        : [{
                    'Subnet'    : self.subnet,
                }],
            },
        }

    def to_service_payload(self):
        return {
            'Target'      : self.id_ or self.name,
        }

    async def create(self, client):
        payload = self.to_payload()
        results = await client.network_create(payload)
        self.id_ = results['Id']
        return results



import arrow
import base64

from .base import DockerObject

class Config(DockerObject):
    "A docker swarm config"
    def __init__(self, name, data, labels=None, id_=None, **kwargs):
        self.id_    = id_
        self.data   = data.encode('utf-8') if isinstance(data, str) else data
        self.labels = labels or {}
        self.name   = name
        for k, v in kwargs.items():
            setattr(self, k, v)

    @staticmethod
    def parse(payload):
        return Config(
            created_at  = arrow.get(payload['CreatedAt']),
            data        = base64.b64decode(payload['Spec']['Data']),
            labels      = payload['Spec']['Labels'],
            id_         = payload['ID'],
            name        = payload['Spec']['Name'],
            updated_at  = arrow.get(payload['UpdatedAt']),
            version     = payload['Version']
        )

    def to_payload(self):
        return {
            'Data'      : base64.b64encode(self.data).decode('utf-8'),
            'Labels'    : self.labels,
            'Name'      : self.name,
        }

    def to_service_payload(self):
        raise Exception("looks like you're trying to specify a Config directly in a service. Use a ConfigMount instead")

    async def create(self, client):
        payload = self.to_payload()
        results = await client.config_create(payload)
        self.id_ = results['ID']
        return results

    async def delete(self, client):
        return await client.config_rm(self.id_)

class ConfigMount(DockerObject):
    "A mount of a Config inside of a Service"
    def __init__(self, config, filename, mode=292, uid="0", gid="0"):
        self.config     = config
        self.gid        = gid
        self.filename   = filename
        self.mode       = mode
        self.uid        = uid

    def to_service_payload(self):
        return {
            'File'          : {
                'Name'      : self.filename,
                'GID'       : self.gid,
                'Mode'      : self.mode,
                'UID'       : self.uid,
            },
            'ConfigID'      : self.config.id_,
            'ConfigName'      : self.config.name,
        }

    @staticmethod
    def parse(payload):
        if not payload:
            return
        configs = [Config(
            name = config['ConfigName'],
            id_  = config['ConfigID'],
            data = None,
        ) for config in payload]
        mounts = [ConfigMount(
            config      = config,
            filename    = mount['File']['Name'],
            gid         = mount['File']['GID'],
            mode        = mount['File']['Mode'],
            uid         = mount['File']['UID'],
        ) for config, mount in zip(configs, payload)]
        return mounts
